@@ -1334,7 +1334,7 @@ pw_filter_new_simple(struct pw_loop *loop,
 	if (props == NULL)
 		return NULL;
 
-	context = pw_context_new(loop, NULL, 0);
+	context = pw_context_new(loop, pw_properties_copy(props), 0);
 	if (context == NULL) {
 		res = -errno;
 		goto error_cleanup;
@@ -1662,7 +1662,7 @@ pw_filter_connect(struct pw_filter *filter,
 		struct spa_fraction q;
 		if (sscanf(str, "%u/%u", &q.num, &q.denom) == 2 && q.denom != 0) {
 			pw_properties_setf(filter->properties, PW_KEY_NODE_FORCE_RATE,
-					"1/%u", q.denom);
+					"%u", q.denom);
 			pw_properties_setf(filter->properties, PW_KEY_NODE_FORCE_QUANTUM,
 					"%u", q.num);
 		}
@@ -1993,6 +1993,14 @@ int pw_filter_get_time(struct pw_filter *filter, struct pw_time *time)
 	return 0;
 }
 
+SPA_EXPORT
+uint64_t pw_filter_get_nsec(struct pw_filter *filter)
+{
+	struct timespec ts;
+	clock_gettime(CLOCK_MONOTONIC, &ts);
+	return SPA_TIMESPEC_TO_NSEC(&ts);
+}
+
 SPA_EXPORT
 struct pw_buffer *pw_filter_dequeue_buffer(void *port_data)
 {
