@@ -607,9 +607,50 @@ static inline void emit_param_changed(struct stream *impl,
 	impl->in_emit_param_changed--;
 }
 
+static void emit_node_info(struct stream *d, bool full)
+{
+	uint32_t i;
+	uint64_t old = full ? d->info.change_mask : 0;
+	if (full)
+		d->info.change_mask = d->change_mask_all;
+	if (d->info.change_mask != 0) {
+		if (d->info.change_mask & SPA_NODE_CHANGE_MASK_PARAMS) {
+			for (i = 0; i < d->info.n_params; i++) {
+				if (d->params[i].user > 0) {
+					d->params[i].flags ^= SPA_PARAM_INFO_SERIAL;
+					d->params[i].user = 0;
+				}
+			}
+		}
+		spa_node_emit_info(&d->hooks, &d->info);
+	}
+	d->info.change_mask = old;
+}
+
+static void emit_port_info(struct stream *d, bool full)
+{
+	uint32_t i;
+	uint64_t old = full ? d->port_info.change_mask : 0;
+	if (full)
+		d->port_info.change_mask = d->port_change_mask_all;
+	if (d->port_info.change_mask != 0) {
+		if (d->port_info.change_mask & SPA_PORT_CHANGE_MASK_PARAMS) {
+			for (i = 0; i < d->port_info.n_params; i++) {
+				if (d->port_params[i].user > 0) {
+					d->port_params[i].flags ^= SPA_PARAM_INFO_SERIAL;
+					d->port_params[i].user = 0;
+				}
+			}
+		}
+		spa_node_emit_port_info(&d->hooks, d->direction, 0, &d->port_info);
+	}
+	d->port_info.change_mask = old;
+}
+
 static int impl_set_param(void *object, uint32_t id, uint32_t flags, const struct spa_pod *param)
 {
 	struct stream *impl = object;
+	struct pw_stream *stream = &impl->this;
 
 	if (id != SPA_PARAM_Props)
 		return -ENOTSUP;
@@ -617,6 +658,11 @@ static int impl_set_param(void *object, uint32_t id, uint32_t flags, const struc
 	if (impl->in_set_param == 0)
 		emit_param_changed(impl, id, param);
 
+	if (stream->state == PW_STREAM_STATE_ERROR)
+		return stream->error_res;
+
+	emit_node_info(impl, false);
+	emit_port_info(impl, false);
 	return 0;
 }
 
@@ -686,46 +732,6 @@ static int impl_send_command(void *object, const struct spa_command *command)
 	return 0;
 }
 
-static void emit_node_info(struct stream *d, bool full)
-{
-	uint32_t i;
-	uint64_t old = full ? d->info.change_mask : 0;
-	if (full)
-		d->info.change_mask = d->change_mask_all;
-	if (d->info.change_mask != 0) {
-		if (d->info.change_mask & SPA_NODE_CHANGE_MASK_PARAMS) {
-			for (i = 0; i < d->info.n_params; i++) {
-				if (d->params[i].user > 0) {
-					d->params[i].flags ^= SPA_PARAM_INFO_SERIAL;
-					d->params[i].user = 0;
-				}
-			}
-		}
-		spa_node_emit_info(&d->hooks, &d->info);
-	}
-	d->info.change_mask = old;
-}
-
-static void emit_port_info(struct stream *d, bool full)
-{
-	uint32_t i;
-	uint64_t old = full ? d->port_info.change_mask : 0;
-	if (full)
-		d->port_info.change_mask = d->port_change_mask_all;
-	if (d->port_info.change_mask != 0) {
-		if (d->port_info.change_mask & SPA_PORT_CHANGE_MASK_PARAMS) {
-			for (i = 0; i < d->port_info.n_params; i++) {
-				if (d->port_params[i].user > 0) {
-					d->port_params[i].flags ^= SPA_PARAM_INFO_SERIAL;
-					d->port_params[i].user = 0;
-				}
-			}
-		}
-		spa_node_emit_port_info(&d->hooks, d->direction, 0, &d->port_info);
-	}
-	d->port_info.change_mask = old;
-}
-
 static int impl_add_listener(void *object,
 		struct spa_hook *listener,
 		const struct spa_node_events *events,
@@ -973,7 +979,8 @@ static int impl_port_use_buffers(void *object,
 		if (SPA_FLAG_IS_SET(impl_flags, PW_STREAM_FLAG_MAP_BUFFERS)) {
 			for (j = 0; j < buffers[i]->n_datas; j++) {
 				struct spa_data *d = &buffers[i]->datas[j];
-				if ((mappable_dataTypes & (1<<d->type)) > 0) {
+				if (SPA_FLAG_IS_SET(d->flags, SPA_DATA_FLAG_MAPPABLE) ||
+				    (mappable_dataTypes & (1<<d->type)) > 0) {
 					if ((res = map_data(impl, d, prot)) < 0)
 						return res;
 					SPA_FLAG_SET(b->flags, BUFFER_FLAG_MAPPED);
@@ -2035,7 +2042,7 @@ pw_stream_connect(struct pw_stream *stream,
 		struct spa_fraction q;
 		if (sscanf(str, "%u/%u", &q.num, &q.denom) == 2 && q.denom != 0) {
 			pw_properties_setf(stream->properties, PW_KEY_NODE_FORCE_RATE,
-					"1/%u", q.denom);
+					"%u", q.denom);
 			pw_properties_setf(stream->properties, PW_KEY_NODE_FORCE_QUANTUM,
 					"%u", q.num);
 		}
@@ -2380,6 +2387,14 @@ int pw_stream_get_time_n(struct pw_stream *stream, struct pw_time *time, size_t
 	return 0;
 }
 
+SPA_EXPORT
+uint64_t pw_stream_get_nsec(struct pw_stream *stream)
+{
+	struct timespec ts;
+	clock_gettime(CLOCK_MONOTONIC, &ts);
+	return SPA_TIMESPEC_TO_NSEC(&ts);
+}
+
 static int
 do_trigger_deprecated(struct spa_loop *loop,
                  bool async, uint32_t seq, const void *data, size_t size, void *user_data)
